'use strict';

var util = require('util');
var EventEmitter = require('events');
var packet = require('./packet');
var Serial = require('./serial');
var Escape = require('./escape');

var MAX_LENGTH = 500;
function Transceiver() {
    EventEmitter.call(this);
    var that = this;
    this._sequence = 0;
    this._parser = new packet.Parser();
    this._build = packet.build;
    this._serial = new Serial();
    this._escape = new Escape();
    this._encode = this._escape.encode.bind(this._escape);
    this._pushUncodedData = this._escape.pushUndecodedData.bind(this._escape);

    this._parser.on('data', function (data, seq) {
        // console.log('checksum ok, seq is ', seq);
        that.emit('data', data);
    });
    this._parser.on('error', function (error, seq) {
        // console.log(error.message, 'seq is ', seq);
        that.emit('error', error);
    });

    this._escape.on('error', function (error) {
        // console.log(error.message);
        that.emit('error', error);
    });
    this._escape.on('data', function (data) {
        that._parser.append(data);
    });

    this._serial.on('data', function (data) {
        that._pushUncodedData(data);
    });

    this._serial.startRead();
}

util.inherits(Transceiver, EventEmitter);

Transceiver.prototype._sendUnit = function (data, callback) {
    var packetData = this._build(this._sequence, data);
    this._sequence++;
    var packetDataEncoded = this._encode(packetData);
    this._serial.write(packetDataEncoded, callback);
};

Transceiver.prototype.send = function (data, callback) {
    var dataPending = data;
    var dataToSend;
    while (dataPending.length > 0) {
        if (dataPending.length <= MAX_LENGTH) {
            dataToSend = dataPending;
            dataPending = new Buffer(0);
        } else {
            dataToSend = dataPending.slice(0, MAX_LENGTH);
            dataPending = dataPending.slice(MAX_LENGTH);
        }
        this._sendUnit(dataToSend);
    }
    callback && callback();
};

module.exports = Transceiver;
